#ifndef ATI_H
#define ATI_H

#include <typedefs.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef UINT32 AHIDEVCONTEXT_T;
typedef UINT32 AHISURFACE_T;
typedef UINT32 AHIDEVICE_T;

typedef enum {
  AHIFMT_1BPP = 0,
  AHIFMT_4bpp,
  AHIFMT_8BPP,
  AHIFMT_16BPP_444,
  AHIFMT_16BPP_555,
  AHIFMT_16BPP_565} AHIPIXFMT_T;

typedef struct
{
  INT32 x, y;

} AHIPOINT_T;

typedef struct
{
  INT32 x1, y1, x2, y2;

} AHIRECT_T;

typedef enum
{
    AHICMP_ALWAYS = 0,
    AHICMP_NEVER,
    AHICMP_EQUAL,
    AHICMP_NOTEQUAL

} AHICMP_T;

typedef struct
{
    UINT32      key;
    UINT32      mask;
    AHICMP_T    cmp;

} AHICOLOROP_T;

typedef struct
{
  UINT8   stridex, stridey;
  UINT8   w, h;
  void    *image;

} AHICHAR_T;

typedef struct
{
  UINT32    width;
  UINT32    height;
  void     *image;
  INT32     stride;
  UINT32    format;

} AHIBITMAP_T;

typedef struct
{
  UINT32      width;
  UINT32      height;
  UINT32      offset;
  INT32       stride;
  UINT32      byteSize;
  AHIPIXFMT_T pixFormat;
  UINT32      numPlanes;

} AHISURFINFO_T;

typedef enum
{
  AHIROT_0 = 0,
  AHIROT_90,
  AHIROT_180,
  AHIROT_270

} AHIROTATE_T;

typedef enum
{
    AHIBLEND_SRC_PLUS_DST = 0,
    AHIBLEND_SRC_MINUS_DST,
    AHIBLEND_DST_MINUS_SRC,
    AHIBLEND_SRC_MINUS_ALPHA,
    AHIBLEND_SRC_SHR1,
    AHIBLEND_DST_PLUS_SRC_DIV2

} AHIBLEND_T;

typedef enum
{
    AHIMIRR_NO = 0,
    AHIMIRR_VERTICAL,
    AHIMIRR_HORIZONTAL,
    AHIMIRR_VER_HOR

} AHIMIRROR_T;

typedef struct
{
  char    drvName[80]; // driver name
  char    drvVer[80]; // driver version
  UINT8   pad[172];

} AHIDRVINFO_T; //sizeof(AHIDRVINFO_T) == 0x14C




/* -----------------------------------    ----------------------------------- */


/*    */
AHIDEVCONTEXT_T DAL_GetDeviceContext( UINT32 displayID );

/*    AHI-   index */
UINT32 AhiDevEnum( AHIDEVICE_T    *dev,   // result
                   AHIDRVINFO_T   *drvInfo, // result
                   UINT32     index );


/*      
    dev - ,  AhiDevEnum
    caller -     */
UINT32 AhiDevOpen( AHIDEVCONTEXT_T  *devCx,   // result
                   AHIDEVICE_T    dev,
                   char       *caller,
                   UINT32     flags );


/*   */
UINT32 AhiDevClose( AHIDEVCONTEXT_T devCx );




/* -----------------------------------    ----------------------------------- */


/*     ,   ,   */
UINT32 AhiSurfGetLargestFreeBlockSize( AHIDEVCONTEXT_T    devCx,
                                       AHIPIXFMT_T      pixFormat,
                                       UINT32       *size,    // result
                                       UINT32       *align,   // result
                                       UINT32       flags );


/*    
    size -   
     flags == 2,      ,  
     . */
UINT32 AhiSurfAlloc( AHIDEVCONTEXT_T  devCx,
                     AHISURFACE_T   *surf,  // result
                     AHIPOINT_T     *size,
                     AHIPIXFMT_T    pixFormat,
                     UINT32       flags );
#define AHIFLAG_DISPLAYPIXFMT 2


/*     */
UINT32 AhiSurfFree( AHIDEVCONTEXT_T   devCx,
                    AHISURFACE_T    surf   );


/*     */
UINT32 AhiSurfInfo( AHIDEVCONTEXT_T   devCx,
                    AHISURFACE_T    surf,
                    AHISURFINFO_T   *info );


/*     */
UINT32 AhiDispSurfGet( AHIDEVCONTEXT_T  devCx,
                       AHISURFACE_T     *surf );  // result


/*     */
UINT32 AhiDispSurfSet( AHIDEVCONTEXT_T  devCx,
                       AHISURFACE_T   surf,
                       UINT32     flags );


/*      */
AHISURFACE_T DAL_GetDrawingSurface( UINT32 displayID );


/*    ()  
     flags == 1,      bitmap */
UINT32 AhiSurfCopy( AHIDEVCONTEXT_T   devCx,
                    AHISURFACE_T    surf,
                    AHIBITMAP_T     *bitmap,
                    AHIRECT_T     *dstRect,
                    AHIPOINT_T      *srcPt,
                    UINT32        param5,
                    UINT32        flags );


/* -----------------------------------    ----------------------------------- */


/*      (  ) */
UINT32 AhiDrawFgColorSet( AHIDEVCONTEXT_T devCx,
                          UINT32      color ); // RGB565 @ E398


/*      (  ) */
UINT32 AhiDrawBgColorSet( AHIDEVCONTEXT_T devCx,
                          UINT32      color );


/*       */
UINT32 AhiDrawBrushFgColorSet( AHIDEVCONTEXT_T  devCx,
                               UINT32     color );


/*       */
UINT32 AhiDrawBrushBgColorSet( AHIDEVCONTEXT_T  devCx,
                               UINT32     color );


/*    :
    pattern -      ( NULL)
    srcPt -    */
UINT32 AhiDrawBrushSet( AHIDEVCONTEXT_T   devCx,
                        AHIBITMAP_T     *pattern,
                        AHIPOINT_T      *srcPt,
                        UINT32        param3,
                        UINT32        flags );
#define AHIFLAG_BRUSH_TRANSPARENT   1
#define AHIFLAG_BRUSH_SOLID       2


/*   - (source)  */
UINT32 AhiDrawSurfSrcSet( AHIDEVCONTEXT_T devCx,
                          AHISURFACE_T    surf,
                          UINT32      flags );


/*   - (destination)  */
UINT32 AhiDrawSurfDstSet( AHIDEVCONTEXT_T devCx,
                          AHISURFACE_T    surf,
                          UINT32          flags );


/*     -.
     clipRect == NULL,       */
UINT32 AhiDrawClipSrcSet( AHIDEVCONTEXT_T devCx,
                          AHIRECT_T     *clipRect );


/*     - */
UINT32 AhiDrawClipDstSet( AHIDEVCONTEXT_T devCx,
                          AHIRECT_T     *clipRect );


/*    (ROP).   AHIROP3
      .
      ROP,   :
    http://msdn.microsoft.com/en-us/library/dd145130(VS.85).aspx
    http://www.svgopen.org/2003/papers/RasterOperationsUsingFilterElements/index.html */
UINT32 AhiDrawRopSet( AHIDEVCONTEXT_T devCx,
                      UINT32      rop );
#define AHIROP_BLACKNESS      0x00
#define AHIROP_NOTSRCERASE    0x11
#define AHIROP_NOTSRCCOPY     0x33
#define AHIROP_SRCERASE       0x44
#define AHIROP_DSTINVERT      0x55
#define AHIROP_PATINVERT      0x5A
#define AHIROP_SRCINVERT      0x66
#define AHIROP_SRCAND         0x88
#define AHIROP_MERGEPAINT     0xBB
#define AHIROP_MERGECOPY      0xC0
#define AHIROP_SRCCOPY        0xCC
#define AHIROP_SRCPAINT       0xEE
#define AHIROP_PATCOPY        0xF0
#define AHIROP_PATPAINT       0xFB
#define AHIROP_WHITENESS      0xFF

#define AHIROP3(_rop) ( (_rop) | ((_rop)<<8) )


/*      AhiDrawAlphaBlt.
    alpha  , RGB888 */
UINT32 AhiDrawAlphaSet( AHIDEVCONTEXT_T   devCx,
                        UINT32        alpha );
#define AHIALPHA(_level) ( (_level) | ((_level)<<8) | ((_level)<<16) )




/* -----------------------------------   ----------------------------------- */


/*   .
    points -    
    count -  
     flags == 0,    ,    
     ("")
     flags == 1,         
    ( count    ) */
UINT32 AhiDrawLines( AHIDEVCONTEXT_T  devCx,
                     AHIPOINT_T       *pointArr,
                     UINT32           count,
                     UINT32           flags );
#define AHIFLAG_LINE_STRIP        0
#define AHIFLAG_LINE_SEPARATE     1


/*     */
UINT32 AhiDrawSpans( AHIDEVCONTEXT_T  devCx,
                     AHIRECT_T        *rectArr,
                     UINT32           count,
                     UINT32           flags );


/*    -  ,   ROP */
UINT32 AhiDrawBitBlt( AHIDEVCONTEXT_T devCx,
                      AHIRECT_T     *dstRect,
                      AHIPOINT_T    *srcPt );


/*  ,  AhiDrawBitBlt,     */
UINT32 AhiDrawBitBltMulti( AHIDEVCONTEXT_T    devCx,
                           AHIRECT_T      *dstRectArr,
                           AHIPOINT_T     *srcPtArr,
                           UINT32       count );


/*       -
    palette -   
     flags == 1,     "0" ()  ,
     flags == 2,   "1" */
UINT32 AhiDrawBitmapBlt( AHIDEVCONTEXT_T  devCx,
                         AHIRECT_T      *dstRect,
                         AHIPOINT_T     *srcPt,
                         AHIBITMAP_T    *bitmap,
                         void       *palette,
                         UINT32       flags );


/*    -      
    maskPt -    
    mask -   (1bpp)
    maskStride -      */
UINT32 AhiDrawMaskBlt( AHIDEVCONTEXT_T  devCx,
                       AHIRECT_T    *dstRect,
                       AHIPOINT_T   *srcPt,
                       AHIPOINT_T   *maskPt,
                       void       *mask,
                       UINT32     maskStride );

/*       */
UINT32 AhiDrawRotateBlt( AHIDEVCONTEXT_T  devCx,
                         AHIRECT_T      *dstRect,
                         AHIPOINT_T     *srcPt,
                         AHIROTATE_T    rotation,
                         AHIMIRROR_T    mirroring,
                         UINT32       flags );


/*      
      AHICOLOROP_T: if( cmp(color & mask, key) ) draw(); */
UINT32 AhiDrawTransBlt( AHIDEVCONTEXT_T   devCx,
                        AHIRECT_T     *dstRect,
                        AHIPOINT_T      *srcPt,
                        AHICOLOROP_T    *dstCop,
                        AHICOLOROP_T    *srcCop );


/*    AhiDrawRotateBlt  AhiDrawTransBlt */
UINT32 AhiDrawSpriteBlt( AHIDEVCONTEXT_T  devCx,
                         AHIRECT_T      *dstRect,
                         AHIPOINT_T     *srcPt,
                         AHICOLOROP_T   *dstCop,
                         AHICOLOROP_T   *srcCop,
                         AHIROTATE_T    rotation,
                         AHIMIRROR_T    mirroring,
                         UINT32       flags );


/*    -    -.
    bitmap -    (4bpp)
     flags == 1,     , 
    AhiDrawAlphaSet */
UINT32 AhiDrawAlphaBlt( AHIDEVCONTEXT_T   devCx,
                        AHIRECT_T     *dstRect,
                        AHIPOINT_T      *srcPt,
                        AHIPOINT_T      *bmpPt,
                        AHIBITMAP_T     *bitmap,
                        UINT32        flags );
#define AHIFLAG_ALPHA_SOLID       1


/*    ()
  flags == 1..4   */
UINT32 AhiDrawStretchBlt( AHIDEVCONTEXT_T devCx,
                          AHIRECT_T     *dstRect,
                          AHIRECT_T     *srcRect,
                          UINT32      flags );


/*     */
UINT32 AhiDrawBlendBlt( AHIDEVCONTEXT_T   devCx,
                        AHIRECT_T     *dstRect,
                        AHIPOINT_T      *srcPt,
                        AHIBLEND_T      blendOp,
                        UINT32        flags );


/*     */
UINT32 AhiDrawTileBlt( AHIDEVCONTEXT_T    devCx,
                       AHIRECT_T      *dstRect,
                       AHIPOINT_T     *srcPt,
                       UINT32       flags );


/*    */
UINT32 AhiDrawChar( AHIDEVCONTEXT_T   devCx,
                    AHIPOINT_T      *dstPt,
                    AHICHAR_T     *charsArr,
                    UINT32        count,
                    UINT32        flags );


/*    */
UINT32 AhiDrawGlyph( AHIDEVCONTEXT_T  devCx,
                     AHIPOINT_T     *dstPt,
                     AHIBITMAP_T    *charsArr,
                     UINT32       count,
                     UINT32       flags );




/* -----------------------------------   ----------------------------------- */


/*   VBlank  */
UINT32 AhiDispWaitVBlank( AHIDEVCONTEXT_T devCx,
                          UINT32      flags );



#ifdef __cplusplus
}
#endif

#endif