#ifndef UTIL_H
#define UTIL_H

#include <typedefs.h>
#include <stdarg.h>
#include <time_date.h>

#ifdef __cplusplus
extern "C" {
#endif

#define NEWLINE "\r\n" //    


/***************************************
   Convert
****************************************/

WCHAR* u_uint64tou(UINT64 value, WCHAR *unicode);
void APP_UtilGetDataVolume (UINT16 feature_id, UINT64 * data_vol);
void DL_DbFeatureGetBlock(UINT16 feature, UINT8* prt );

// unicode to ascii (valid only for ascii-range symbols) 
char* u_utoa( const WCHAR* ustr,  char* astr );
// ascii  to unicode 
WCHAR* u_atou( char* astr,  WCHAR* ustr );

// Unicode to UTF-8
UINT16 DL_Char_convUCS2toUTF8String( WCHAR *src_ptr,
                  UINT16  src_len,
                  UINT8   *trg_ptr,
                  UINT16  trg_len);   
// UTF-8 to Unicode
UINT16 DL_Char_convUTF8toUCS2String(UINT8 *src_ptr,
                  UINT16  src_len, 
                  WCHAR   *trg_ptr, 
                  UINT16  trg_len);

// CP-1251 to Unicode                 
UINT16 DL_Char_convCP1251toUCS2String(UINT8   *src_ptr,
                    UINT16  src_len,
                    WCHAR   *trg_ptr,
                    UINT16  trg_len);
// KOI8-R to Unicode
UINT16 DL_Char_convKOI8RtoUCS2String(UINT8   *src_ptr,
                   UINT16  src_len,
                   WCHAR   *trg_ptr,
                   UINT16  trg_len);
               
/***************************************
   Unicode strings routines
****************************************/

// strcpy  ,  src (   NULL)  ,  dst.
WCHAR* u_strcpy( WCHAR* dst,  WCHAR* src ); 
//  n   src  dst 
WCHAR* u_strncpy(WCHAR* dst, WCHAR* src, UINT32 n); 

//   src   dst
WCHAR* u_strcat( WCHAR* dst,  WCHAR* src );
//  n   src   dst
WCHAR* u_strncat(WCHAR* dst, WCHAR* src, UINT32 n);   

//  
UINT32 u_strlen( const WCHAR* str );

//   str1  str2,
INT32 u_strcmp( WCHAR* str1,  WCHAR* str2 ); //  ,  0
INT32 u_strncmp( WCHAR* str1,  WCHAR* str2,  UINT32 n );

//     
WCHAR* u_strmakelower( WCHAR* str );
//   
WCHAR* u_strmakeupper( WCHAR* str );

//    
WCHAR* u_ltou( INT32 value,  WCHAR* str );
//    
INT32 u_atol( WCHAR* str );

//   str    chr
WCHAR* u_strchr(WCHAR* src, WCHAR chr); 
//   str    chr
WCHAR* u_strrchr(WCHAR* src, WCHAR chr); 

//  search  src (  )
WCHAR* u_strstr(WCHAR* src, WCHAR* search); 
// ,    
WCHAR* u_stristr(WCHAR* src, WCHAR* search); 

//   dec  hex
WCHAR* u_itoh(INT32 value, WCHAR* hex_buf); 

//     n    pos
INT32 u_strcut(WCHAR *str, UINT16 len_str, UINT16 pos, INT32 n); 

/***************************************
   ANSI strings routines
****************************************/

// Quick compare strings 
UINT32 namecmp( char* str1,  char* str2 ); //returns 1 if equal

char* strcpy( char* dst,  char* src );  //   (n   strncpy),    <src> (   ),  ,    <dest>.
char* strncpy( char* dst, char* src, UINT32 n );  // . 

char* strcat( char* dst,  char* src );  //   <src> (n   strncat)   <dst>,       <dst>      .
char* strncat( char* dst, char* src, UINT32 n );  //  
                    
//   (n   strncmp) : s1  s2.   ,  ,     ,  s1  ,    s2.
INT32 strcmp( char* str1,  char* str2 );       
INT32 strncmp( char* str1,  char* str2,  UINT32 n );


//   str  long
INT32 strtol( char* str,  char** endstr,  int radix ); 
//   unsigned long
UINT32 strtoul( char* str,  char** endstr,  int radix ); 
//    
char* _itoa(int value, char *string, int radix);
#define itoa(v,s,r)   _itoa(v,s,r)

//  
UINT32 strlen( char* str ); 

//    
char*  strchr( const char* s, int c); 
//     ()
char*  strrchr( char* s, int c);  
//  
char*  strstr( const char *, const char * );
//    
char*  strtok( char* s1, char* s2);

int toupper( int c );   //    
int tolower( int c );   //    

UINT32 strcspn( char* str1, char* str2 );   //     str1       str2
UINT32 strspn( char* str1, char* str2 );    // ,  ``  

//    
void *bsearch( const void *key, const void *buf, UINT32 num, UINT32 size, int (*compare)(const void *, const void *) );
// 
void qsort( void *buf, UINT32 num, UINT32 size, int (*compare)(const void *, const void *) );

//           format
int vsprintf (char *buffer, const char *format, va_list arglist); 
//    buffer  ,  arguments
int Nsscanf(char *buffer, char *format, ... /*arguments*/);
#define sscanf(buffer, format, ...)   Nsscanf (buffer, format, ## __VA_ARGS__)



/***************************************
    
****************************************/

UINT32 rand( void); //  
void srand(UINT32 seed); //  

#define random(num)     (rand() % (num))
#define randomize()     srand((UINT32)suPalReadTime())

/***************************************
  MATH
****************************************/

#define max(a,b)        (((a) > (b)) ? (a) : (b))
#define min(a,b)        (((a) < (b)) ? (a) : (b))

#define abs(x)      ((x) < 0 ? -(x) : (x))
int abs_0( int );

/*
//   
#define degree(x, y) {  \
    if (y == 0) {  \
         *x = 1;  \
    } else {  \
        while (y > 1) {  \
            *x *= *x; \
            y--;  \
        }  \
    }  \
}
*/

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
