#ifndef DSMA_H
#define DSMA_H

#ifndef DL_EV_BASE
#define DL_EV_BASE				0x82000
#endif

#define EV_DSMA_PROXY_CREATE	DL_EV_BASE | 0x01B1
#define EV_DSMA_PROXY_STATE		DL_EV_BASE | 0x01B2
#define EV_DSMA_BUFF_STATE		DL_EV_BASE | 0x01B3
#define EV_DSMA_PROXY_ATTR		DL_EV_BASE | 0x01B4


// protocols
#define PROTOCOL_TCP		1
#define PROTOCOL_SSL		2
#define PROTOCOL_UDP		3
#define PROTOCOL_IP			4

#define USE_CSD				1
#define USE_GPRS			2


#define PROXY_NULL			0


typedef UINT32		CMD_NUM;


// proxy_state
enum {
    PROXY_STATE_ACTIVE,		// 
    PROXY_STATE_INACTIVE,	//  (  )
    PROXY_STATE_FAILED		// 
};

// buf_state
enum {
    BUFFER_STATE_UNK1,
    BUFFER_STATE_UNK2,
    BUFFER_STATE_DATA		//  
};

enum {
	DSMA_ATTR_NULL = 0,
	
	DSMA_ATTR_TRANS_PROT = 0x1000,
	DSMA_ATTR_LOCAL_PORT,
	DSMA_ATTR_REMOTE_PORT,
	DSMA_ATTR_PRIORITY,
	
	DSMA_ATTR_NET_PROT = 0x2000,
	DSMA_ATTR_LOCAL_IP,
	DSMA_ATTR_REMOTE_IP,
	
	DSMA_ATTR_LINK_PROT = 0x3000,
	DSMA_ATTR_USER_NAME,
	DSMA_ATTR_PW,
	DSMA_ATTR_UNKNOWN,
	DSMA_ATTR_DNS_1,
	DSMA_ATTR_DNS_2,
	
	DSMA_ATTR_PHYS_PROT = 0x4000,
	DSMA_ATTR_APN
};


typedef struct {
    UINT16		attr; // DSMA_ATTR_*
    UINT32		unk1;
    void *		value;
    UINT32		size;
    UINT32		result;
} DSMA_ATTRIBUTE_T;


typedef struct {
    UINT8		unk1;

    UINT8		unk2;
    UINT8		unk3;
    UINT8		unk4;

    UINT8		gprs_err;
    UINT8		err;
    UINT8		unk5;
} DSMA_STRUCT_1; 

typedef struct {
    UINT8 unk1;
    UINT8 unk2;
    UINT8 unk3;
} DSMA_STRUCT_2; 

typedef struct {
    UINT8 unk1;
    UINT16 unk2;
    UINT8 unk3;
} DSMA_STRUCT_3; 


typedef struct {
    UINT32				        proxy_id;
    UINT8			            status;			//   
    CMD_NUM                     cmd;
    SU_PORT_T		            port;			//     ( )  
    UINT8						proxy_state;
    UINT8						buf_state;
    UINT32						attr_num;
    DSMA_ATTRIBUTE_T  *			attr_list;
    UINT32						lp_handle;
    DSMA_STRUCT_1				unk1;
    DSMA_STRUCT_2				unk2;
    UINT32						unk3;
	
	
	#if (defined(L7e) || defined(L9))
	UINT8						sessionId;
	DSMA_STRUCT_3				unk4;
	UINT8						unk5;
	UINT32						*localPortInfo;
	DSMA_STRUCT_3				unk6;
	#endif
	
	
 } DSMA_DATA_T;  
 
	
 

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

//     CMD_NUM     CMD_NUM          

//  ,    EV_DSMA_PROXY_CREATE
CMD_NUM
DL_DSMA_ProxyCreate( SU_PORT_T port );

// / 
CMD_NUM
DL_DSMA_ProxyDestroy( UINT32 proxy_id );

//  ,   EV_DSMA_PROXY_ATTR
CMD_NUM
DL_DSMA_ProxyAttributeSet( UINT32 proxy_id, UINT32 attr_count, DSMA_ATTRIBUTE_T * attr_list );

//  ,   EV_DSMA_PROXY_ATTR
CMD_NUM 
DL_DSMA_ProxyAttributeGet( UINT32 proxy_id, UINT32 attr_count, DSMA_ATTRIBUTE_T * attr_list );

//  ,      ,   EV_DSMA_PROXY_STATE
CMD_NUM
DL_DSMA_ProxyActivate( UINT32 proxy_id );

//  
CMD_NUM
DL_DSMA_ProxyDeactivate( UINT32 proxy_id );

//  ,        
//    /,    gprs ,    -
CMD_NUM
DL_DSMA_ProxyDeactivateSocket( UINT32 proxy_id );

//  ,  - 
UINT32
DL_DSMA_ProxyWrite( UINT32 proxy_id, BYTE * buf, UINT32 size );

//  ,  - 
UINT32
DL_DSMA_ProxyRead( UINT32 proxy_id, BYTE * buf, UINT32 size );



#ifdef __cplusplus
}
#endif /* __cplusplus */ 

#endif