#ifndef EVENTS_H
#define EVENTS_H

#include <typedefs.h>

#ifdef __cplusplus
extern "C" {
#endif

// . EVENT_T  EVENT_DATA_T @apps.h

#define EV_GRANT_TOKEN            30
#define EV_REVOKE_TOKEN           31

#define EV_KEY_PRESS              500   // key_pressed
#define EV_KEY_RELEASE            501   // key_pressed

//      GUI 
#define EV_VOL_CHANGE_REQUEST     0x20CA


//     
#define EV_DIALOG_DONE            0x201C
//      ""
#define EV_DONE                   0x2020
//     /  
#define EV_LIST_NAVIGATE          0x2043 // index
//         ""
#define EV_SELECT                 0x2058 // index
//       
#define EV_REQUEST_LIST_ITEMS     0x202C // list_items_req
//  
#define EV_NO                     0x2046
#define EV_YES                    0x2079
// 
#define EV_CANCEL         0x2008 

//  
#define EV_DATA                   0x200F // EVENT_T::attachment
// ?
#define EV_BROWSE                 0x2006

//     - 
#define EV_DATA_CHANGE            0x2011 // index
//  
#define EV_REQUEST_DATA           0x202B // data_req

//    
extern const UINT32 *EV_IDLE_GAIN_FOCUS;
#define EV_IDLE_FOCUS   *EV_IDLE_GAIN_FOCUS//0x76A

//  
#define EV_GAIN_FOCUS       20 
//  
#define EV_LOSE_FOCUS         21 

#define EV_RENDER           25
//  
#define EV_USER_ACTIVITY      0x7F1
//   
#define EV_REG_NETWORK          0x8200B 
//  
#define EV_POWER_DOWN       0x11

//  
#define EV_CREATE_SHORTCUT      0x20C5  

//  
extern const UINT32 *CALL_ALERT;  //    
#define EV_CALL_ALERT     *CALL_ALERT

//  
extern const UINT16 *ALMCLK_REMINDER_START; //    
#define EV_ALMCLK_REMINDER_START *ALMCLK_REMINDER_START;

//  
extern const UINT32 *DATEBOOK_REMINDER;
#define EV_DATEBOOK_REMINDER  *DATEBOOK_REMINDER;

//   
extern const UINT32 *EV_DisplayTimeout;
#define EV_DISPLAY_NO_ACTIVE  *EV_DisplayTimeout

//  
extern const UINT32 *EV_DisplayActivity;
#define EV_DISPLAY_ACTIVE *EV_DisplayActivity

//11111111111111111111111111111111111111111111111

// e   
extern const UINT32 *EV_BacklightContinueOn;
#define EV_DISPLAY_NOT_OFF *EV_BacklightContinueOn

// e   
extern const UINT32 *EV_BacklightRestore;
#define EV_DISPLAY_AUTO_OFF *EV_BacklightRestore

//   
extern const UINT32 *EV_LOC_KPAD;
#define EV_KEYPAD_LOCK      *EV_LOC_KPAD
//  
extern const UINT32 *EV_UNLOC_KPAD;
#define EV_KEYPAD_UNLOCK    *EV_UNLOC_KPAD

extern const UINT32 *EV_MsgNotif; //   
#define EV_MSG_NOTIF    *EV_MsgNotif

extern const UINT32 *EV_MSGG_PHBK_SEARCH;
#define EV_PHBK_SEARCH  *EV_MSGG_PHBK_SEARCH //    

extern const UINT32 *EV_CALL_INCOMING;   //   
#define EV_CALLS_INCOMING *EV_CALL_INCOMING

extern const UINT32 *EV_CALL_TERMINATED; //   
#define EV_CALLS_TERMINATED *EV_CALL_TERMINATED

//  
extern const UINT32 *EV_MISSEDCALLS;
#define EV_CALLS_MISSED *EV_MISSEDCALLS

// / ,   id  (UINT8)
#define EV_DEVICE_ATTACH  0x822F3 //  
#define EV_DEVICE_DETACH  0x822F4 //  

//-     
extern void APP_MMC_UtilStopVariousTimers(void); 
//-     
extern void APP_MMC_UtilRestartVariousTimers(void); 


typedef enum
{
      FBF_FREE = 100,  //    
      FBF_LEAVE        //      
} FREE_BUF_FLAG_T;

typedef struct
{
    UINT32                    bufsize;
    UINT32                    index;
    UINT8                     unk2;
} REQUEST_DATA_T;

typedef struct
{
    UINT32                    ID;           // ?  
    UINT32                    begin_idx;    //     
    UINT8                     count;        //    
} REQUEST_LIST_ITEMS_T;

typedef struct
{ 
  UINT32          list_idx;
  UINT32          unk1;
  void *          ptr;
//  void *          func_ptr;
} DATA_CHANGE_DATA_T;

/* ,   ,    */
typedef union
{
  UINT8           key_pressed;
  UINT16            value16;
  UINT32            value32;
  UINT32            index;
  UINT32            params[16];
  REQUEST_LIST_ITEMS_T    list_items_req;
  REQUEST_DATA_T        data_req;
  DATA_CHANGE_DATA_T      data_change;

  //ELF parameters
  ELF_PARAMS_T        start_params;
  //padding to real size
  UINT8           pad[0xFB];
} EVENT_DATA_T;

typedef struct
{
    UINT32                    data_tag;
    EVENT_DATA_T              data;
} ADD_EVENT_DATA_T;

/*    */
typedef struct tagEVENT_T
{
    EVENT_CODE_T              code;
    UINT32                    unk1;
    UINT32                    ev_port;
    UINT8                     unk2;
    BOOL                      is_consumed;
    BOOL                      is_firstpass;
    UINT8                     unk3;
    UINT32                    unk4;
    UINT32                    unk5;
    struct tagEVENT_T        *next;
    UINT32                    data_tag;
    EVENT_DATA_T              data;
    BOOL                      is_attach;
    UINT16                    att_size;
    void                      *attachment;
    UINT8                     unk6;
} EVENT_T; //size = 296


typedef struct
{
    EVENT_CODE_T              code;
    UINT8                     unk1;
    UINT8                     unk2;
    EVENT_T                   *top;
    void                      *unk3;
    UINT8                     unk4[6];
    UINT8                     unk5;
} EVENT_STACK_T;

//   event  
EVENT_T* AFW_GetEv( EVENT_STACK_T *ev_st );

//    (     )
UINT32 AFW_AddEvNoD( EVENT_STACK_T    *ev_st,  
                     UINT32           event_code );

UINT32 AFW_AddEvEvD( EVENT_STACK_T    *ev_st,
                     UINT32           event_code,
                     ADD_EVENT_DATA_T *data );

UINT32 AFW_AddEvAux( EVENT_STACK_T    *ev_st,
                     UINT32           event_code,
                     FREE_BUF_FLAG_T  free_buf,
                     UINT32           att_size,
                     void             *attachment );

UINT32 AFW_AddEvAuxD( EVENT_STACK_T     *ev_st,
                      UINT32            event_code,
                      ADD_EVENT_DATA_T  *data,
                      FREE_BUF_FLAG_T  free_buf,
                      UINT32           att_size,
                      void             *attachment );
            
//                 
UINT32 AFW_TranslateEvEvD( EVENT_STACK_T    *ev_st,
                           UINT16           event_code,
                           ADD_EVENT_DATA_T *data );              

//     
UINT32 APP_ConsumeEv( EVENT_STACK_T *ev_st,  void *app );

//   Internal . .      
UINT32 AFW_CreateInternalQueuedEvAux( UINT32 event_code, 
                    FREE_BUF_FLAG_T  free_buf, 
                    UINT32 att_size, 
                    void* att_data ); 

UINT32 AFW_CreateInternalQueuedEvAuxD( UINT32 event_code, 
                     void *data, 
                     FREE_BUF_FLAG_T  free_buf, 
                     UINT32 size, 
                     void *aux_data);


//   EV_LIST_ITEMS (  EV_REQUEST_LIST_ITEMS)
UINT32 APP_UtilAddEvUISListData( EVENT_STACK_T    *ev_st,
                                 void             *app,
                                 UINT32           param2, // = 0
                                 UINT32           begin_index,
                                 UINT8            count,
                                 FREE_BUF_FLAG_T  free_buf,
                                 UINT32           bufsize,
                                 void             *buffer );

//   EV_REQUEST_DATA     
UINT32 APP_UtilAddEvUISGetDataForItem( EVENT_STACK_T    *ev_st,
                                       void             *app,
                                       UINT32           index ); 

//  
UINT32 APP_UtilAddEvUISListChange( EVENT_STACK_T    *ev_st,
                                   void             *app,
                                   UINT32           param2, // = 0
                                   UINT32           position,
                                   UINT8            size,
                                   BOOL             refresh_data,
                                   UINT8            param6,          // = 2
                                   FREE_BUF_FLAG_T  free_buf,
                                   UINT32           bufsize,  // NULL ?
                                   void             *buffer );  // NULL ?
//     
UINT32 APP_UtilAddEvChangeListPosition ( EVENT_STACK_T    *p_evg,
                     void             *app,
                                         UINT32           position,
                     FREE_BUF_FLAG_T  free_buf,
                     UINT32           bufsize,
                     void             *buffer );               
 
#ifdef __cplusplus
} /* extern "C" */
#endif
 
#endif
