#ifndef MME_H
#define MME_H

#include <typedefs.h>


#ifdef __cplusplus
extern "C" {
#endif

#ifdef L7e
#define EV_MME_RANGE_START		  0xE0000
#else 
	#ifdef L9
	#define EV_MME_RANGE_START		  0xE0000
	#else
	#define EV_MME_RANGE_START		  0xC1000
	#endif
#endif


#define EV_MME_OPEN_SUCCESS       EV_MME_RANGE_START //   
#define EV_MME_OPEN_ERROR         EV_MME_RANGE_START+0x1 //    
#define EV_MME_SEEK_SUCCESS       EV_MME_RANGE_START+0x2 //    - MME_GC_playback_seek
#define EV_MME_SEEK_ERROR         EV_MME_RANGE_START+0x3 //   seek
#define EV_MME_PLAY_COMPLETE      EV_MME_RANGE_START+0xD //     //   PLAY_COMPLETE_T
#define EV_MME_CLOSE_COMPLETE     EV_MME_RANGE_START+0x14 //  
#define EV_MME_STOP_COMPLETE      EV_MME_RANGE_START+0x15 //  
#define EV_MME_STOPPED_AT_TIME    EV_MME_RANGE_START+0x16 //   ?



typedef   void*    MME_GC_MEDIA_FILE;

typedef struct
{
   IFACE_DATA_T                 iface_data;
   MME_GC_MEDIA_FILE            media_handle;
   UINT8						status; // = 1 //    ,  L7e - 128
} MME_PLAY_COMPLETE_T; 

typedef struct
{
   IFACE_DATA_T          iface_data;
   MME_GC_MEDIA_FILE   	 media_handle;
} MME_OPEN_SUCCESS_T;

/*******************************/

typedef enum
{
	// 
	MIME_TYPE_IMAGE_GIF = 0,
	MIME_TYPE_IMAGE_BMP,
	MIME_TYPE_IMAGE_WBMP,
	MIME_TYPE_IMAGE_PNG,
	MIME_TYPE_IMAGE_JPEG,

	//  
	MIME_TYPE_AUDIO_MID = 11,
	MIME_TYPE_AUDIO_MIDI,
	MIME_TYPE_AUDIO_MIX,
	MIME_TYPE_AUDIO_BAS,
	MIME_TYPE_AUDIO_MP3,
	MIME_TYPE_AUDIO_AAC,
	MIME_TYPE_AUDIO_AMR = 24,
	MIME_TYPE_AUDIO_MP4 = 26,
	MIME_TYPE_AUDIO_M4A,

	// 
	MIME_TYPE_VIDEO_ASF = 34,
	MIME_TYPE_VIDEO_MP4 = 36,
	MIME_TYPE_VIDEO_MPEG4,
	MIME_TYPE_VIDEO_H263

} MIME_TYPE_T;

typedef struct 
{
    WCHAR           *str;
    UINT16          str_size;
} STRING_T;

typedef struct 
{
    WCHAR           *str;
    UINT16          str_size;
    UINT8           unk0[4];
    UINT8           unk1[4];
} RATING_T;

typedef struct 
{
    STRING_T   title;
    STRING_T   author;
    STRING_T   copyright;
    STRING_T   description;
    STRING_T   performer;
    STRING_T   genre;
    RATING_T   rating;
    STRING_T   location;
    STRING_T   date;
    STRING_T   album;

} MEDIA_FILE_INFO_T;

typedef enum 
{
   PLAY_RATE_1X = 1,
   PLAY_RATE_2X,
   PLAY_RATE_3X,
   PLAY_RATE_4X,
   PLAY_RATE_MAX
} PLAY_RATE_T;

typedef enum{
    CODEC_NONE=0,
	MP3_CODEC=2,
    AAC_CODEC,
    MIDI_CODEC,
    WAV_CODEC,
    AMR_CODEC=15
} AUDIO_CODEC_TYPE_T;

typedef struct
{
    UINT32        				bit_rate;
    AUDIO_CODEC_TYPE_T          audio_codec;
    UINT32        				sampling_freq;
    UINT8           			audio_mode;    // 0=Mono, 1=Stereo 
    UINT16       				unk1[7];
    UINT8            			wav_bit_rate; // 0 =4, 1=8, 2=16 Kb/s
    UINT16            			unk2[7];
} AUDIO_FORMAT_T;

#ifdef L7e
	#define ATTRIBUTE_RANGE_1 	0x5C //  L7e  Z3
#else
	#ifdef L9
		#define ATTRIBUTE_RANGE_1 	0x5E //  L9  K1 
	#else
		#define ATTRIBUTE_RANGE_1 	71
	#endif
#endif

typedef enum
{
  // get  set
	FILE_INFO=0, 				// MEDIA_FILE_INFO_T
	DURATION,    				// UINT32
	DURATION_MS, 				// UINT32
	FILE_SIZE,					// UINT32
	MEDIA_PATH=6,				// WCHAR
	PLAY_RATE=8,				// PLAY_RATE_T
	MEDIA_STOP_TIME=58,			// UINT32
	MEDIA_STOP_TIME_MS=60,		// UINT32
	PLAYBACK_AUDIO_VOLUME=63, 	// UINT8
	MEDIA_VOLUME,				// UINT8		
  
   // get
    AUDIO_FORMAT=ATTRIBUTE_RANGE_1,		// AUDIO_FORMAT_T
	POSITION=ATTRIBUTE_RANGE_1 + 3,		// UINT32
	POSITION_MS,							// UINT32 
	PAUSE_POSITION=114						// UINT32
} ATTRIBUTE_NAME_T;

typedef struct RECTANGLE_LIST
{
   UINT16                  x1; //   
   UINT16                  y1;
   UINT16                  x2; //   
   UINT16                  y2; 
   struct RECTANGLE_LIST   *next_list;
} RECTANGLE_LIST_T;

typedef struct  
{
   UINT16                  frame_num;
   RECTANGLE_LIST_T   *rectangle_list;
} SAFE_FRAME_ID_T; //     


typedef enum
{
    // Video 
    MMSS_MP4=3,
    MMSS_3GP,
    MMSS_ROM,
    MMSS_RAW,
   
    // Audio 
    MMSS_WMAT=12, 
    MMSS_MP3,
    MMSS_MIDI,
    MMSS_IMELODY,
    MMSS_AMR=19,
    MMSS_AAC,
    MMSS_WAV=22,

    // Image 
    MMSS_JPEG=25,
    MMSS_JFIF,
    MMSS_GIF=30,
    MMSS_PNG,
    MMSS_BMP,
    MMSS_WBMP,
    MMSS_EMS_BMP

} MMSS_FILE_FORMAT_T;

typedef enum
{
    H263_BASELINE_CODEC=1,
    H263_PROFILE_3_CODEC,
    WMV_CODEC,
    WMV9_CODEC,
    RV8_CODEC,
    RV9_CODEC,
    MPEG4_LEVEL_0_CODEC, // default
    VIDEO_NANCY_CODEC,
    VIDEO_INPUT_DEVICE  //   

} VIDEO_CODEC_T; 

typedef enum
{
  COUNT_COLOR_2=1, 
  COUNT_COLOR_4,
  COUNT_COLOR_8, 
  COUNT_COLOR_16,
  COUNT_COLOR_32,
  COUNT_COLOR_64,
  COUNT_COLOR_128,
  COUNT_COLOR_256,
  COUNT_COLOR_512,
  COUNT_COLOR_1024,
  COUNT_COLOR_2048,
  COUNT_COLOR_4096,
  COUNT_COLOR_8192,
  COUNT_COLOR_16384,
  COUNT_COLOR_32768,
  COUNT_COLOR_65536 // 16bit, default

} COUNT_COLORS_T;

typedef enum
{
    COUNT_FRAME_RATE_1=1,
    COUNT_FRAME_RATE_2,
    COUNT_FRAME_RATE_3,
    COUNT_FRAME_RATE_4,
    COUNT_FRAME_RATE_5,
    COUNT_FRAME_RATE_6,
    COUNT_FRAME_RATE_7,
    COUNT_FRAME_RATE_10,
    COUNT_FRAME_RATE_15 // default

} COUNT_VIDEO_FRAME_RATE_T;

typedef struct MMSS_VIDEO_FORMAT_S
{
    VIDEO_CODEC_T          		video_codec; // -
    COUNT_COLORS_T              count_colors;
    UINT32				        bit_rate;
    UINT8					    unk1; // =2
    COUNT_VIDEO_FRAME_RATE_T    frame_rate; // -   
	UINT32    					frame_width; //  
    UINT32   					frame_height; //  
} VIDEO_FORMAT_T;

typedef enum
{
    JPEG_HEIRARCHICAL_CODEC=1,
    GIF87a_CODEC,
    GIF89a_CODEC,
    JPEG_PROGRESSIVE_CODEC,
    JPEG_BASELINE_CODEC // default
} IMAGE_CODEC_T;

typedef struct
{
    COUNT_COLORS_T          colors;
    UINT8   				unk1; // = 9 ???
    IMAGE_CODEC_T       	image_codec;
    UINT8   				unk2; // = 0,1,2 ???
} IMAGE_FORMAT_T;

typedef enum
{
    MMSS_AUDIO,
    MMSS_VIDEO,
    MMSS_AUDIO_VIDEO,
    MMSS_IMAGE
} MMSS_MEDIA_TYPE_T;

typedef struct
{
    MMSS_MEDIA_TYPE_T               media_type;
    AUDIO_FORMAT_T             		audio_format;
    VIDEO_FORMAT_T             		video_format;
    IMAGE_FORMAT_T       			image_format;
} MMSS_MEDIA_FORMAT_T;

typedef struct 
{
    UINT16   width;
    UINT16   height;
} PICTURE_SIZE_T;



/************
   Playback
  *************/

//   uri  ,      . uri      "file:/",    
MME_GC_MEDIA_FILE  MME_GC_playback_create(
	IFACE_DATA_T  		*data, 
	WCHAR 				*uri,
	SAFE_FRAME_ID_T 	*sf_id, //    
	INT32 				unk1, 
	void 				*image_mem, //   
	UINT32				image_size,
	UINT32 				unk4, // = 0 -for audio&video, =1 - for image  
	void 				*unk5, 
	PICTURE_SIZE_T		*picture_size //  
); 

//  
UINT32 MME_GC_playback_delete(MME_GC_MEDIA_FILE  media_file); 

//   
UINT32 MME_GC_playback_start(MME_GC_MEDIA_FILE  media_file, UINT32 unk1, UINT32 unk2); 

//   
UINT32 MME_GC_playback_stop(MME_GC_MEDIA_FILE  media_file); 

// 
UINT32 MME_GC_playback_pause(MME_GC_MEDIA_FILE  media_file); 

//   ms .   
UINT32 MME_GC_playback_seek(MME_GC_MEDIA_FILE  media_file, UINT32 ms); 


//   
UINT32 MME_GC_playback_get_attribute( MME_GC_MEDIA_FILE     handle,
									  ATTRIBUTE_NAME_T   		attribute_name,
								      void  				*attribute_value
									);

//   
UINT32 MME_GC_playback_set_attribute( MME_GC_MEDIA_FILE		handle,
									  ATTRIBUTE_NAME_T   	attribute_name,
									  void  				*attribute_value
									);
									
/*****************
   Capiture
*****************/

//    
MME_GC_MEDIA_FILE MME_GC_capture_create
(
    IFACE_DATA_T                  *iface_data,
    SAFE_FRAME_ID_T               *sf_id, //   - NULL
    MMSS_FILE_FORMAT_T            file_format, //  
    MMSS_MEDIA_FORMAT_T     	  *media_format
);

//   
UINT32 MME_GC_capture_delete(MME_GC_MEDIA_FILE media_handle);

//  
UINT32 MME_GC_capture_start
(
    MME_GC_MEDIA_FILE   media_handle,
    WCHAR   			*path //   
);

//  
UINT32 MME_GC_capture_set_attribute
(
   MME_GC_MEDIA_FILE           media_handle,
   ATTRIBUTE_NAME_T   		   attribute_name,
   void                        *attribute_value
);

//  
UINT32 MME_GC_capture_stop(MME_GC_MEDIA_FILE media_handle);

//  
UINT32 MME_GC_capture_pause(MME_GC_MEDIA_FILE media_handle);


//  
UINT32 MME_GC_capture_get_attribute
(
    MME_GC_MEDIA_FILE           media_handle,
   ATTRIBUTE_NAME_T   		   attribute_name,
   void                        *attribute_value
);

#ifdef __cplusplus
} /* extern "C" */
#endif									
									
#endif
